from prince.ca import CA
from prince.famd import FAMD
from prince.mca import MCA
from prince.pca import PCA
from prince.svd import SVD

__title__ = 'prince'
__version__ = '0.2.0'
__author__ = 'Max Halford'
__license__ = 'MIT'
__copyright__ = 'Copyright 2016 Max Halford'
