from collections import OrderedDict


GRAYS = OrderedDict([
    ('light', '#bababa'),
    ('dark', '#404040')
])

SEABORN = OrderedDict([
    ('blue', '#4c72b0'),
    ('green', '#55a868'),
    ('red', '#c44e52'),
    ('yellow', '#ccb974'),
    ('purple', '#8172b2'),
    ('cyan', '#64b5cd')
])
